"""Test the dict conversion."""

import pytest

from palabox.utils.case_conversion import (
    camelcase_to_snake_case,
    normal_to_camelcase,
    normal_to_snake_case,
    snake_case_to_camelcase,
)


@pytest.mark.parametrize(
    "test_input,expected",
    [("abc", "abc"), ("ABC", "abc"), ("abc def", "abc_def"), ("Abc Def", "abc_def")],
)
def test_normal_to_snake_case(test_input: str, expected: str) -> None:
    """normal_to_camelcase."""
    result = normal_to_snake_case(test_input)
    assert isinstance(result, str)
    assert result == expected


@pytest.mark.parametrize(
    "test_input,expected",
    [("abc", "abc"), ("ABC", "abc"), ("abc def", "abcDef"), ("Abc Def", "abcDef")],
)
def test_normal_to_camelcase(test_input: str, expected: str) -> None:
    """normal_to_camelcase."""
    result = normal_to_camelcase(test_input)
    assert isinstance(result, str)
    assert result == expected


TESTS = [("abc", "abc"), ("abcDef", "abc_def")]


@pytest.mark.parametrize(
    "test_input,expected",
    TESTS,
)
def test_camelcase_to_snake_case(test_input: str, expected: str) -> None:
    """camelcase_to_snake_case."""
    result = camelcase_to_snake_case(test_input)
    assert isinstance(result, str)
    assert result == expected


@pytest.mark.parametrize(
    "test_input,expected",
    [(y, x) for x, y in TESTS],
)
def test_snake_case_to_camelcase(test_input: str, expected: str) -> None:
    """snake_case_to_camelcase."""
    result = snake_case_to_camelcase(test_input)
    assert isinstance(result, str)
    assert result == expected



"""Test the cleaning of html."""

import pytest

from palabox.scraping.clean_html import clean_html_to_text

TEST_CASES = [
    ("<p>ABC</p>", "ABC"),
    ("<p>ABC</p><p>ABC</p>", "ABC\nABC"),
    ("<h1>ABC</h1><p>ABC</p>", "ABC\nABC"),
    ("<p>ABC</p><h1>ABC</h1>", "ABC\n\n\nABC"),
]


@pytest.mark.parametrize("test_input,expected", TEST_CASES)
def test_clean_html_to_text(test_input: str, expected: str) -> None:
    """Test clean_html_to_text."""
    result = clean_html_to_text(f"<html><body>{test_input}</body></html>")
    assert isinstance(result, str)
    assert result == expected



"""CodinGame stats."""

import json
from dataclasses import asdict

import dacite
import requests

from palabox.code_stats.codingame.types import CodinGameStats


def get_user_stats(userid: str) -> CodinGameStats:
    """Get user stats."""
    response = requests.post(
        "https://www.codingame.com/services/CodinGamer/findCodingamePointsStatsByHandle",
        json=[userid],
    )
    if response.ok:
        data: CodinGameStats = dacite.from_dict(CodinGameStats, response.json())
        return data

    raise ValueError(f"Problem with the request {response}, '{response.content.decode()}'")


if __name__ == "__main__":
    data = get_user_stats("8c4c1d3cf92a07fd755428fbc86744fc2575773")
    with open("test.json", "w", encoding="utf-8") as file:
        json.dump(asdict(data), file, indent=2)



"""Types."""

from dataclasses import dataclass
from datetime import datetime
from typing import List


@dataclass
class GrandTotal:
    """GrandTotal."""

    hours: int
    minutes: int
    total_seconds: float


@dataclass
class Range:
    """Range."""

    date: str
    end: datetime
    start: datetime
    text: str
    timezone: str


@dataclass
class RangeStats:
    """Stats for one range of data."""

    grand_total: GrandTotal
    range: Range


@dataclass
class WakatimeStats:
    """Wakatime stats."""

    data: List[RangeStats]



"""Cut properly some text."""

import re

END_OF_SENTENCE_CHARACTERS = {".", ";", "!", "?"}


def properly_cut_text(
    text: str, start_idx: int, end_idx: int, nbr_before: int = 30, nbr_after: int = 30
) -> str:
    """Properly cut a text around some interval."""
    str_length = len(text)
    start_idx = max(0, start_idx - nbr_before)
    end_idx = end_idx + nbr_after

    # Change the end depending on the value
    match = re.search(r"\.[^\d]|\?|\!", text[end_idx:], flags=re.IGNORECASE)
    if match:
        end_idx = match.end() + end_idx
    else:
        end_idx = str_length

    # Change the beginning depending on the value
    match = re.search(r"(\.|\?|\!)(?!.*\1)", text[: start_idx - 1], flags=re.IGNORECASE)
    if match:
        start_idx = match.end() + 1
    else:
        start_idx = 0

    return text[start_idx:end_idx].strip()



"""Config file."""

import os


# pylint: disable=too-few-public-methods
class Config:
    """Configuration object."""

    ROOT_FOLDER = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..")



"""Code Stats."""

from palabox.code_stats.codingame.main import get_user_stats as get_codingame
from palabox.code_stats.leetcode.main import get_user_stats as get_leetcode
from palabox.code_stats.wakatime.main import get_user_stats as get_wakatime

__all__ = ["get_codingame", "get_wakatime", "get_leetcode"]



"""Case conversion."""

from typing import Any, Dict

from .case_conversion import camelcase_to_snake_case


def convert_dict_keys_to_snake_case(dictionary: Dict[str, Any]) -> Dict[str, Any]:
    """Convert a dict to keys."""
    return {
        camelcase_to_snake_case(key): convert_dict_keys_to_snake_case(value)
        if isinstance(value, dict)
        else value
        for key, value in dictionary.items()
    }



"""Logging functions and elements."""

import logging
from typing import Any

import colorlog


def setup_logging(log_level: int) -> None:
    """Setup the logging config."""

    handler = colorlog.StreamHandler()
    handler.setFormatter(
        colorlog.ColoredFormatter(
            "%(asctime)s [pid %(process)d] %(log_color)s%(levelname)s: %(message)s",
            datefmt="%m/%d/%Y %I:%M:%S",
        )
    )

    logging.basicConfig(
        level=log_level,
        handlers=[handler],
    )


class DisableLogger:
    """Disable the logging for a few lines."""

    current_state: int

    def __enter__(self) -> None:
        self.current_state = logging.root.level
        logging.disable(logging.CRITICAL)

    def __exit__(self, exit_type: Any, exit_value: Any, exit_traceback: Any) -> None:
        logging.disable(self.current_state - 1)



"""Clean a html page."""

import re
from typing import Union

from bs4 import BeautifulSoup
from bs4.element import NavigableString, Tag

TAGS_TO_REMOVE = [
    "header",
    "head",
    "footer",
    "nav",
    "script",
    "noscript",
    "style",
    "iframe",
    "svg",
    "input",
    "button",
    "textarea",
    "button",
    "checkbox",
    "label",
]
TAGS_HEADERS = {"h" + str(i) for i in range(6)}
TAGS_FORCE_NEWLINE = {"tbody", "thead", "section", "p"}
TAGS_SPACE = {"a", "strong", "i"}
TAGS_TAB = {"th", "td"}

TO_STRIP = "\t\n "


def get_text_recursive(tag: Union[Tag, NavigableString, None]) -> str:
    """Extract the text using the childrens."""
    if tag is None:
        return ""
    if isinstance(tag, NavigableString):
        return str(tag).strip().replace("\n", " ")

    tag_name = tag.name

    # Special tags, like headers, lists, tables, ...
    if tag_name in TAGS_HEADERS:
        return (
            "\n" * 2
            + " ".join(
                filter(
                    lambda x: len(x) > 0,
                    [get_text_recursive(x) for x in tag.children if x is not None],
                )
            )
            + "\n"
        )
    if tag_name in {"ul", "ol"}:
        return (
            "\n".join(
                map(
                    lambda x: "- " + x,
                    filter(
                        lambda x: len(x) > 0,
                        [get_text_recursive(x) for x in tag.children if x is not None],
                    ),
                )
            )
            + "\n" * 2
        )
    if tag_name == "tr":
        return (
            "- "
            + ";  ".join(
                filter(
                    lambda x: len(x) > 0,
                    [get_text_recursive(x) for x in tag.children if x is not None],
                ),
            )
            + "\n" * 2
        )

    # What should be the end, after the text of the element
    if tag_name in TAGS_FORCE_NEWLINE:
        end = "\n"
    elif tag_name in TAGS_SPACE:
        end = " "
    else:
        end = ""

    return (
        " ".join(
            filter(
                lambda x: len(x) > 0, [get_text_recursive(x) for x in tag.children if x is not None]
            )
        )
        + end
    )


def clean_html_to_text(raw_html: str) -> str:
    """Extract the meaningful text from the page."""
    # Preprocessing
    raw_html = re.sub(r"\<\!\-\-.*\-\-\>", "", raw_html)
    raw_html = re.sub(r"[”“”]", '"', raw_html)
    soup = BeautifulSoup(raw_html, features="html.parser").find("body")

    # Clean the soup before boiling/cooking it
    for to_remove in (
        soup.find_all(TAGS_TO_REMOVE)
        + soup.find_all(class_=re.compile(r"(header|footer)"))
        + soup.find_all(id=re.compile(r"(header|footer)"))
    ):
        to_remove.extract()

    # Get all the paragraphs / texts needed in the selected tags
    text = ""

    for tag in soup.find_all(recursive=False):
        if tag is None:
            continue
        text += get_text_recursive(tag)

    # Special processing
    text = re.sub(r"\n[ \t]+", "\n", text)
    text = re.sub(r" +", r" ", text)
    text = text.strip()

    return text



"""Case conversion."""

import re


def normal_to_snake_case(text: str) -> str:
    """Convert from normal to snake_case."""
    return text.lower().replace(" ", "_")


def normal_to_camelcase(text: str) -> str:
    """Convert from normal to camelCase."""
    texts = text.lower().split(" ")
    texts = [texts[0]] + list(map(lambda x: x.capitalize(), texts[1:]))
    return "".join(texts)


def camelcase_to_snake_case(text: str) -> str:
    """Convert from camelCase to snake_case."""
    return re.sub(r"([A-Z])", r"_\g<1>", text).lower()


def snake_case_to_camelcase(text: str) -> str:
    """Convert from snake_case to camelCase."""
    return normal_to_camelcase(text.replace("_", " "))



from typing import Dict

import pytest

from palabox.utils.dict_conversion import convert_dict_keys_to_snake_case

TESTS = [
    ({"abc": 1}, {"abc": 1}),
    ({"abcDef": 1}, {"abc_def": 1}),
    ({"abcDef": {"efJi": "ef_ji"}, "x": 1}, {"abc_def": {"ef_ji": "ef_ji"}, "x": 1}),
]


@pytest.mark.parametrize(
    "test_input,expected",
    TESTS,
)
def test_convert_dict_keys_to_snake_case(test_input: Dict, expected: Dict) -> None:
    """normal_to_camelcase."""
    result = convert_dict_keys_to_snake_case(test_input)
    assert str(result) == str(expected)



"""Datatypes."""
# pylint: disable=invalid-name,too-many-instance-attributes

from dataclasses import dataclass
from typing import List, Optional


@dataclass
class QuestionCount:
    """QuestionCount."""

    difficulty: str
    count: int


@dataclass
class Contributions:
    """Contributions."""

    points: int
    questionCount: int
    testcaseCount: int


@dataclass
class Profile:
    """Profile."""

    realName: str
    websites: List[str]
    countryName: str
    skillTags: List[str]
    company: Optional[str]
    school: Optional[str]
    starRating: float
    aboutMe: str
    userAvatar: str
    reputation: int
    ranking: int


@dataclass
class SubmissionCount:
    """SubmissionCount."""

    difficulty: str
    count: int
    submissions: int


@dataclass
class SubmitStats:
    """SubmitStats."""

    acSubmissionNum: List[SubmissionCount]
    totalSubmissionNum: List[SubmissionCount]


@dataclass
class MatchedUser:
    """MatchedUser."""

    username: str
    githubUrl: str
    contributions: Contributions
    profile: Profile
    submitStats: SubmitStats


@dataclass
class LeetCodeData:
    """Data from LeetCode about one user."""

    allQuestionsCount: List[QuestionCount]
    matchedUser: MatchedUser



"""Scraping for serverside rendering."""

from typing import Optional

import requests

from .clean_html import clean_html_to_text


def scrap_one_website(url: str, session: Optional[requests.Session] = None) -> str:
    """Scrap one website."""
    if session is None:
        content: str = requests.get(url).content.decode()
    else:
        content = session.get(url).content.decode()
    return content


if __name__ == "__main__":
    _ = scrap_one_website("https://marco.boucas.fr/")
    with open("test.txt", "w", encoding="utf-8") as file:
        file.write(_)
    _ = clean_html_to_text(_)
    with open("test2.txt", "w", encoding="utf-8") as file:
        file.write(_)



"""Wakatime stats."""

import json
from dataclasses import asdict
from datetime import datetime

import dacite
import requests

from palabox.code_stats.wakatime.types import WakatimeStats
from palabox.utils import json_converter


def get_user_stats(token: str) -> WakatimeStats:
    """Get user stats."""
    response = requests.get(
        f"https://wakatime.com/share/{token}.json",
    )
    if response.ok:
        data: WakatimeStats = dacite.from_dict(
            WakatimeStats,
            response.json(),
            dacite.Config(
                {datetime: lambda dateStr: datetime.strptime(dateStr, "%Y-%m-%dT%H:%M:%S%z")}
            ),
        )
        return data

    raise ValueError(f"Problem with the request {response}, '{response.content.decode()}'")


if __name__ == "__main__":
    data = get_user_stats(
        "@5fba56dd-c3e1-4bec-9596-fd1565702df9/b026f6b2-5e5f-4a7e-8d1b-8039ee9a6aa6"
    )
    with open("test.json", "w", encoding="utf-8") as file:
        json.dump(asdict(data), file, indent=2, default=json_converter)



"""Datatypes."""
# pylint: disable=invalid-name,too-many-instance-attributes

from dataclasses import dataclass
from typing import List


@dataclass
class FormValues:
    """FormValues."""

    city: str
    school: str


@dataclass
class CodinGamer:
    """CodinGamer."""

    userId: int
    pseudo: str
    countryId: str
    publicHandle: str
    schoolId: int
    rank: int
    onlineSince: int
    formValues: FormValues
    city: str
    level: int
    xp: int
    category: str


@dataclass
class RankHistorics:
    """RankHistorics."""

    ranks: List[int]
    totals: List[int]
    points: List[int]
    contestPoints: List[int]
    optimPoints: List[int]
    codegolfPoints: List[int]
    multiTrainingPoints: List[int]
    clashPoints: List[int]
    dates: List[int]


@dataclass
class CodingamePointsRankingDto:
    """CodingamePointsRankingDto."""

    codingamePointsTotal: int
    codingamePointsRank: int
    codingamePointsContests: int
    codingamePointsAchievements: int
    codingamePointsXp: int
    codingamePointsOptim: int
    codingamePointsCodegolf: int
    codingamePointsMultiTraining: int
    codingamePointsClash: int
    numberCodingamers: int
    numberCodingamersGlobal: int
    rankHistorics: RankHistorics


@dataclass
class CodinGameStats:
    """CodinGameStats."""

    codingamerPoints: int
    achievementCount: int
    codingamer: CodinGamer
    codingamePointsRankingDto: CodingamePointsRankingDto



"""Test properly_cut_text."""

import pytest

from palabox.processing.text.properly_cut_text import properly_cut_text

TEST_CASES = [
    ("My name is Marco.", 0, 17, "My name is Marco."),
    ("My name is Marco.", -1, 17, "My name is Marco."),
    ("My name is Marco.", 0, 20, "My name is Marco."),
]

TEXT_CASES_2 = [
    ("123456789", 2, 5, 1, 2, "123456789"),
    ("123456 789", 2, 5, 1, 2, "123456 789"),
    ("This is a 3.5 dollars bread", 1, 2, 0, 0, "This is a 3.5 dollars bread"),
    ("This is a 3. and 5", 1, 2, 0, 0, "This is a 3."),
    ("This is a 3. and 5", 1, 2, 0, 10, "This is a 3. and 5"),
    ("This is a 3. and 5", 15, 16, 0, 0, "and 5"),
]


@pytest.mark.parametrize("test_input,start_idx,end_idx,expected", TEST_CASES)
def test_properly_cut_text(test_input: str, start_idx: int, end_idx: int, expected: str) -> None:
    """Test properly_cut_text."""
    result = properly_cut_text(test_input, start_idx, end_idx)
    assert result == expected


@pytest.mark.parametrize("test_input,start_idx,end_idx,nbr_before,nbr_after,expected", TEXT_CASES_2)
def test_properly_cut_text_2(
    test_input: str,
    start_idx: int,
    end_idx: int,
    expected: str,
    nbr_before: int,
    nbr_after: int,
) -> None:
    """Test properly_cut_text."""
    result = properly_cut_text(test_input, start_idx, end_idx, nbr_before, nbr_after)
    assert result == expected



"""json_converter."""

from datetime import datetime
from typing import Any


def json_converter(element: Any) -> Any:
    """Converter of elements to JSONable objects."""
    if isinstance(element, datetime):
        return str(element)
    return str(element)



"""Setup.py file."""

import setuptools

with open("README.md", "r", encoding="utf-8") as file:
    long_description = file.read()

with open("requirements.txt", "r", encoding="utf-8") as file:
    requirements = file.read().split("\n")

setuptools.setup(
    name="palabox",  # This is the name of the package
    version="0.0.4",  # The initial release version
    author="Marco Boucas",  # Full name of the author
    url="https://github.com/marcoboucas/palabox",
    description="Toolbox of useful functions, with focus on Text Analysis and Stats",
    long_description=long_description,  # Long description read from the the readme file
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(
        include=["palabox", "palabox.*"]
    ),  # List of all python modules to be installed
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],  # Information to filter the project on PyPi website
    python_requires=">=3.6",  # Minimum version requirement of the package
    py_modules=["palabox"],  # Name of the python package
    install_requires=requirements,  # Install other dependencies if any
)



"""LeetCode stats."""

import json
from dataclasses import asdict

import dacite
import requests

from palabox.code_stats.leetcode.types import LeetCodeData


def get_user_stats(username: str) -> LeetCodeData:
    """Get user stats."""
    # pylint: disable=line-too-long
    response = requests.get(
        "https://leetcode.com/graphql",
        json={
            "operationName": "getUserProfile",
            "variables": {"username": username},
            "query": "query getUserProfile($username: String!) {\n  allQuestionsCount {\n    difficulty\n    count\n    __typename\n  }\n  matchedUser(username: $username) {\n    username\n    socialAccounts\n    githubUrl\n    contributions {\n      points\n      questionCount\n      testcaseCount\n      __typename\n    }\n    profile {\n      realName\n      websites\n      countryName\n      skillTags\n      company\n      school\n      starRating\n      aboutMe\n      userAvatar\n      reputation\n      ranking\n      __typename\n    }\n    submissionCalendar\n    submitStats: submitStatsGlobal {\n      acSubmissionNum {\n        difficulty\n        count\n        submissions\n        __typename\n      }\n      totalSubmissionNum {\n        difficulty\n        count\n        submissions\n        __typename\n      }\n      __typename\n    }\n    badges {\n      id\n      displayName\n      icon\n      creationDate\n      __typename\n    }\n    upcomingBadges {\n      name\n      icon\n      __typename\n    }\n    activeBadge {\n      id\n      __typename\n    }\n    __typename\n  }\n}\n",
        },
    )
    if response.ok:
        data: LeetCodeData = dacite.from_dict(LeetCodeData, response.json()["data"])
        return data
    raise ValueError(f"Problem with the request {response}, '{response.content.decode()}'")


if __name__ == "__main__":
    data = get_user_stats("MarcoBoucas")
    with open("test.json", "w", encoding="utf-8") as file:
        json.dump(asdict(data), file, indent=2)
